/*
 * Copyright (C) 2007 by egnite Software GmbH
 * Copyright (c) 1992, 1993 The Regents of the University of California
 * Copyright (c) 1988 Stephen Deering.
 *
 * This code is derived from software contributed to Berkeley by
 * Stephen Deering of Stanford University
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * \file net/igmpout.c
 * \brief IGMP output functions.
 *
 * \verbatim
 * $Id: igmpout.c 3683 2011-12-04 13:42:04Z haraldkipp $
 * \endverbatim
 */

#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ipcsum.h>
#include <netinet/igmp.h>
#include <string.h>

/*!
 * \addtogroup xgIGMP
 */
/*@{*/
/*!
 * \brief Send an IGMP datagram.
 *
 * Known IGMP types are:
 *
 * - #IGMP_MEMBERSHIP_QUERY Membership query.
 * - #IGMP_V1_MEMBERSHIP_REPORT Version 1 membership report.
 * - #IGMP_V2_MEMBERSHIP_REPORT Version 2 membership report.
 * - #IGMP_V2_LEAVE_GROUP Leave-group message.
 *
 * \param type IGMP packet type.
 * \param dest Destination IP address.
 * \param nb   Network buffer structure containing the datagram.
 *
 * \return 0 on success, -1 otherwise.
 */
int NutIgmpOutput(uint8_t type, uint32_t dest, NETBUF * nb)
{
    IGMP *igmp;

    if ((nb = NutNetBufAlloc(nb, NBAF_TRANSPORT, sizeof(IGMP))) == 0) {
        return -1;
    }
    igmp = (IGMP *) nb->nb_tp.vp;
    nb->nb_tp.sz = sizeof(IGMP);
    igmp->igmp_type = type;
    igmp->igmp_code = 0;
    igmp->igmp_group = dest;
    igmp->igmp_cksum = 0;
    igmp->igmp_cksum = NutIpChkSum(0, nb->nb_tp.vp, nb->nb_tp.sz);

    return NutIpOutput(IPPROTO_IGMP, dest, nb);
}

/*@}*/
